// make certain to include the T3DLIB1.CPP source file!

// INCLUDES ///////////////////////////////////////////////

#define WIN32_LEAN_AND_MEAN  

// you must #define INITGUID if not done elsewhere
#define INITGUID

#include <windows.h>   // include important windows stuff
#include <windowsx.h> 
#include <mmsystem.h>
#include <objbase.h>
#include <iostream> // include important C/C++ stuff
#include <conio.h>
#include <stdlib.h>
#include <malloc.h>
#include <memory.h>
#include <string.h>
#include <stdarg.h>
#include <stdio.h> 
#include <math.h>
#include <io.h>
#include <fcntl.h>


#include <ddraw.h>     // directX includes
#include "T3DLIB1.H"  // include the library header
#include "T3DLIB1.CPP"

//#include <dsound.h>

// DEFINES ////////////////////////////////////////////////

// defines for windows 
#define WINDOW_CLASS_NAME "WINCLASS1"

// default screen size
#define SCREEN_WIDTH    640  // size of screen
#define SCREEN_HEIGHT   480
#define SCREEN_BPP      8    // bits per pixel

#define BITMAP_ID            0x4D42 // universal id for a bitmap
#define MAX_COLORS_PALETTE   256

// skelaton directions
#define SKELATON_EAST         0
#define SKELATON_NEAST        1  
#define SKELATON_NORTH        2
#define SKELATON_NWEST        3
#define SKELATON_WEST         4
#define SKELATON_SWEST        5
#define SKELATON_SOUTH        6
#define SKELATON_SEAST        7


#define START_GLOW_COLOR  152 // starting color index to glow
#define END_GLOW_COLOR    159 // ending color index to glow
#define FLOOR_COLOR       116
#define WALL_COLOR		  1

#define JUMP_TIMER			30 // The amount of clock cycles player will rise when jump trigger is activated
#define	BULLET_TIMER		5
#define GRAVITY				5  // The lower this number the lower the height of a jump will be
#define HORIZONTAL_SPEED    4  // The higher this number the faster the player will move horizontally
#define BULLET_SPEED		8  
#define TOTAL_BULLETS       30
#define TOTAL_ENEMIES		49
#define PLAYER_HEALTH		3
#define BACKGROUNDS			18// Put the number of landscapes here
#define STARTPAGE	    	0
#define BOSS_BULLETS		3
#define SHOOT_DELAY		    25


// PROTOTYPES /////////////////////////////////////////////
void newEnemy(int id, int type, int x, int y, int page, int health);
int Game_Init(void *parms=NULL, int num_parms = 0);
int Game_Shutdown(void *parms=NULL, int num_parms = 0);
int Game_Main(void *parms=NULL,  int num_parms = 0);

// GLOBALS ////////////////////////////////////////////////

// windows vars
HWND      main_window_handle = NULL; // globally track main window
int       window_closed      = 0;    // tracks if window is closed
HINSTANCE main_instance      = NULL; // globally track hinstance

char buffer[80];                // used to print text
int shoot_timer = 0;
int boss_shoot_timer = 0;
// demo globals
BOB          skelaton;     // the player skelaton
BOB			 bullet[TOTAL_BULLETS];
BOB			 enemy[TOTAL_ENEMIES + BOSS_BULLETS];
int game_over_msg = 0;
int bullet_timer = BULLET_TIMER;
bool bullet_reset = true;
int checkpoint_page = 0;
int player_lives = 3;
bool reset_help = true;
bool show_help = false;
int enemy_anims[8][4] = { {0,1,0,2},
                             {0+4,1+4,0+4,2+4},
                             {0+8,1+8,0+8,2+8},
                             {0+12,1+12,0+12,2+12},
                             {0+16,1+16,0+16,2+16},
                             {0+20,1+20,0+20,2+20},
                             {0+24,1+24,0+24,2+24},
                             {0+28,1+28,0+28,2+28}, };
int bullet_anims[8][4] = { {0,1,0,2},
                             {0+4,1+4,0+4,2+4},
                             {0+8,1+8,0+8,2+8},
                             {0+12,1+12,0+12,2+12},
                             {0+16,1+16,0+16,2+16},
                             {0+20,1+20,0+20,2+20},
                             {0+24,1+24,0+24,2+24},
                             {0+28,1+28,0+28,2+28}, };


int skelaton_anims[8][4] = { {0,1,0,2},
                             {0+4,1+4,0+4,2+4},
                             {0+8,1+8,0+8,2+8},
                             {0+12,1+12,0+12,2+12},
                             {0+16,1+16,0+16,2+16},
                             {0+20,1+20,0+20,2+20},
                             {0+24,1+24,0+24,2+24},
                             {0+28,1+28,0+28,2+28}, };

BITMAP_IMAGE landscape[BACKGROUNDS];      // the background  
BITMAP_IMAGE intro;
int curr_page = 0;                        // current page of landscape

// PROTOTYPES //////////////////////////////////////////////

LRESULT CALLBACK WindowProc(HWND hwnd, 
						    UINT msg, 
                            WPARAM wparam, 
                            LPARAM lparam)
{
// this is the main message handler of the system
PAINTSTRUCT		ps;		// used in WM_PAINT
HDC				hdc;	// handle to a device context
char buffer[80];        // used to print strings

// what is the message 
switch(msg)
	{	
	case WM_CREATE: 
        {


		// do initialization stuff here
        // return success
		return(0);
		} break;
   
	case WM_PAINT: 
		{
		// simply validate the window 
   	    hdc = BeginPaint(hwnd,&ps);	 
        
        // end painting
        EndPaint(hwnd,&ps);

        // return success
		return(0);
   		} break;

	case WM_DESTROY: 
		{

		// kill the application, this sends a WM_QUIT message 
		PostQuitMessage(0);

        // return success
		return(0);
		} break;

	default:break;

    } // end switch

// process any messages that we didn't take care of 
return (DefWindowProc(hwnd, msg, wparam, lparam));

} // end WinProc

// WINMAIN ////////////////////////////////////////////////

int WINAPI WinMain(	HINSTANCE hinstance,
					HINSTANCE hprevinstance,
					LPSTR lpcmdline,
					int ncmdshow)
{

WNDCLASSEX winclass; // this will hold the class we create
HWND	   hwnd;	 // generic window handle
MSG		   msg;		 // generic message
HDC        hdc;      // graphics device context

// first fill in the window class stucture
winclass.cbSize         = sizeof(WNDCLASSEX);
winclass.style			= CS_DBLCLKS | CS_OWNDC | 
                          CS_HREDRAW | CS_VREDRAW;
winclass.lpfnWndProc	= WindowProc;
winclass.cbClsExtra		= 0;
winclass.cbWndExtra		= 0;
winclass.hInstance		= hinstance;
winclass.hIcon			= LoadIcon(NULL, IDI_APPLICATION);
winclass.hCursor		= LoadCursor(NULL, IDC_ARROW); 
winclass.hbrBackground	= (HBRUSH)GetStockObject(BLACK_BRUSH);
winclass.lpszMenuName	= NULL;
winclass.lpszClassName	= WINDOW_CLASS_NAME;
winclass.hIconSm        = LoadIcon(NULL, IDI_APPLICATION);

// save hinstance in global
main_instance = hinstance;

// register the window class
if (!RegisterClassEx(&winclass))
	return(0);

// create the window
if (!(hwnd = CreateWindowEx(NULL,                  // extended style
                            WINDOW_CLASS_NAME,     // class
						    "Epic Bunny Rampage", // title
						    WS_POPUP | WS_VISIBLE,
					 	    0,0,	  // initial x,y
						    SCREEN_WIDTH,SCREEN_HEIGHT,  // initial width, height
						    NULL,	  // handle to parent 
						    NULL,	  // handle to menu
						    hinstance,// instance of this application
						    NULL)))	// extra creation parms
return(0);

// save main window handle
main_window_handle = hwnd;

// initialize game here
Game_Init();

// enter main event loop
while(TRUE)
	{
    // test if there is a message in queue, if so get it
	if (PeekMessage(&msg,NULL,0,0,PM_REMOVE))
	   { 
	   // test if this is a quit
       if (msg.message == WM_QUIT)
           break;
	
	   // translate any accelerator keys
	   TranslateMessage(&msg);

	   // send the message to the window proc
	   DispatchMessage(&msg);
	   } // end if
    
       // main game processing goes here
       Game_Main();
       
	} // end while

// closedown game here
Game_Shutdown();

// return to Windows like this
return(msg.wParam);

} // end WinMain

// GAME PROGRAMMING CONSOLE FUNCTIONS ////////////////

int Game_Init(void *parms,  int num_parms)
{
// this function is where you do all the initialization 
// for your game

int index;         // looping var
char filename[80]; // used to build up files names

PlaySound("sounds/techno.wav", main_instance, SND_ASYNC | SND_LOOP);

// initialize directdraw
DDraw_Init(SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_BPP);

// load in each page of the scrolling background
for (index = 0; index<BACKGROUNDS; index++)
    {
    // build up file name
    sprintf(filename,"IMAGES/LANDSCAPE%d.BMP",index+1);

    // load the background
    Load_Bitmap_File(&bitmap8bit, filename);

    // create and load the reactor bitmap image
    Create_Bitmap(&landscape[index], 0,0, 640, 480);
    Load_Image_Bitmap(&landscape[index],&bitmap8bit,0,0,BITMAP_EXTRACT_MODE_ABS);
    Unload_Bitmap_File(&bitmap8bit);

    } // end for index

// set the palette to background image palette
Set_Palette(bitmap8bit.palette);


// now let's load in all the frames for the skelaton!!!

// create skelaton bob
if (!Create_BOB(&skelaton,0,0,72,74,32, // 56x72
           BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_ANIM,DDSCAPS_SYSTEMMEMORY))
   return(0);

// load the frames in 8 directions, 4 frames each
// each set of frames has a walk and a fire, frame sets
// are loaded in counter clockwise order looking down
// from a birds eys view or the x-z plane
for (int direction = 0; direction < 5; direction++)
    { 
    // build up file name
    sprintf(filename,"IMAGES/QUENSP%d.BMP",direction); // skelsp

    // load in new bitmap file
    Load_Bitmap_File(&bitmap8bit,filename);

 
    Load_Frame_BOB(&skelaton,&bitmap8bit,0+direction*4,0,0,BITMAP_EXTRACT_MODE_CELL);  
    Load_Frame_BOB(&skelaton,&bitmap8bit,1+direction*4,1,0,BITMAP_EXTRACT_MODE_CELL);  
    Load_Frame_BOB(&skelaton,&bitmap8bit,2+direction*4,2,0,BITMAP_EXTRACT_MODE_CELL);  
    Load_Frame_BOB(&skelaton,&bitmap8bit,3+direction*4,0,1,BITMAP_EXTRACT_MODE_CELL);  

    // unload the bitmap file
    Unload_Bitmap_File(&bitmap8bit);

    // set the animation sequences for skelaton
    Load_Animation_BOB(&skelaton,direction,4,skelaton_anims[direction]);

    } // end for direction 

// set up stating state of skelaton
Set_Animation_BOB(&skelaton, 1);
Set_Anim_Speed_BOB(&skelaton, 4);
Set_Vel_BOB(&skelaton, 0,0);
Set_Pos_BOB(&skelaton, 16, 200); // right above the floor
skelaton.health = PLAYER_HEALTH;

for(int bullet_count = 0; bullet_count < TOTAL_BULLETS; bullet_count++)
{
	// create skelaton bob
	if (!Create_BOB(&bullet[bullet_count],0,0,72,74,32, // 56x72
			   BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_ANIM,DDSCAPS_SYSTEMMEMORY))
	   return(0);

	// load the frames in 8 directions, 4 frames each
	// each set of frames has a walk and a fire, frame sets
	// are loaded in counter clockwise order looking down
	// from a birds eys view or the x-z plane
	for (direction = 0; direction < 8; direction++)
		{ 
		// build up file name
		sprintf(filename,"IMAGES/Plasma.BMP",direction); // skelsp

		// load in new bitmap file
		Load_Bitmap_File(&bitmap8bit,filename);

		bullet[bullet_count].width = 18;
		bullet[bullet_count].height = 16;
 
		Load_Frame_BOB(&bullet[bullet_count],&bitmap8bit,0+direction*4,0,0,BITMAP_EXTRACT_MODE_CELL);  
		Load_Frame_BOB(&bullet[bullet_count],&bitmap8bit,1+direction*4,1,0,BITMAP_EXTRACT_MODE_CELL);  
		Load_Frame_BOB(&bullet[bullet_count],&bitmap8bit,2+direction*4,2,0,BITMAP_EXTRACT_MODE_CELL);  
		Load_Frame_BOB(&bullet[bullet_count],&bitmap8bit,3+direction*4,0,1,BITMAP_EXTRACT_MODE_CELL);  

		// unload the bitmap file
		Unload_Bitmap_File(&bitmap8bit);

		// set the animation sequences for bullet[bullet_count]
		Load_Animation_BOB(&bullet[bullet_count],direction,4,bullet_anims[direction]);

		} // end for direction

	// set up stating state of bullet
	Set_Animation_BOB(&bullet[bullet_count], 1);
	Set_Anim_Speed_BOB(&bullet[bullet_count], 4);
	Set_Vel_BOB(&bullet[bullet_count], 0,0);
	Set_Pos_BOB(&bullet[bullet_count], 16, 320); // right above the floor
	bullet[bullet_count].health = 0;
	////end test
}


newEnemy(0, 0, 500, 320, 1, 3);
newEnemy(1, 0, 500, 220, 2, 3);
newEnemy(2, 0, 450, 200, 3, 3);
newEnemy(3, 0, 350, 200, 3, 3);
newEnemy(4, 0, 550, 320, 4, 3);
newEnemy(5, 0, 500, 100, 4, 3);
newEnemy(6, 0, 350, 320, 4, 3);
newEnemy(7, 1, 500, 100, 5, 3);
newEnemy(9, 1, 400, 50, 5, 3);
newEnemy(11, 1, 600, 75, 5, 3);
newEnemy(13, 0, 600, 200, 6, 3);
newEnemy(14, 1, 500, 100, 9, 3);
newEnemy(16, 1, 400, 50, 9, 3);
newEnemy(18, 1, 600, 75, 9, 3);
newEnemy(20, 1, 300, 0, 9, 3);
newEnemy(21, 0, 500, 100, 10, 3);

newEnemy(22, 1, 500, 100, 11, 3);
newEnemy(24, 1, 400, 50, 12, 3);
newEnemy(26, 1, 600, 75, 12, 3);
newEnemy(28, 1, 300, 0, 13, 3);
newEnemy(30, 1, 550, 0, 13, 3);
newEnemy(32, 1, 550, 100, 14, 3);
newEnemy(34, 1, 450, 50, 14, 3);
newEnemy(36, 1, 650, 75, 14, 3);
newEnemy(38, 1, 350, 0, 14, 3);
newEnemy(40, 1, 650, 0, 14, 3);
newEnemy(42, 0, 700, 100, 16, 3);
newEnemy(43, 0, 600, 200, 16, 3);
newEnemy(44, 0, 800, 300, 16, 3);
newEnemy(45, 1, 700, 50, 16, 3);
newEnemy(47, 3, 570, 300, 17, 3);
for(int x = 0; x < TOTAL_ENEMIES; x++)
{
	if(enemy[x].type == 0)
	{
		// create skelaton bob
		if (!Create_BOB(&enemy[x],0,0,72,74,32, // 56x72
				   BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_ANIM,DDSCAPS_SYSTEMMEMORY))
		   return(0);

		// load the frames in 8 directions, 4 frames each
		// each set of frames has a walk and a fire, frame sets
		// are loaded in counter clockwise order looking down
		// from a birds eys view or the x-z plane
		for (direction = 0; direction < 8; direction++)
			{ 
			// build up file name
			sprintf(filename,"IMAGES/squirrel%d.BMP",direction); // skelsp
			
			enemy[x].height = 38;
			enemy[x].width = 72;
			// load in new bitmap file
			Load_Bitmap_File(&bitmap8bit,filename);
			Load_Frame_BOB(&enemy[x],&bitmap8bit,0+direction*4,0,0,BITMAP_EXTRACT_MODE_CELL);  
			Load_Frame_BOB(&enemy[x],&bitmap8bit,1+direction*4,1,0,BITMAP_EXTRACT_MODE_CELL);  
			Load_Frame_BOB(&enemy[x],&bitmap8bit,2+direction*4,2,0,BITMAP_EXTRACT_MODE_CELL);  
			Load_Frame_BOB(&enemy[x],&bitmap8bit,3+direction*4,0,1,BITMAP_EXTRACT_MODE_CELL);  

			// unload the bitmap file
			Unload_Bitmap_File(&bitmap8bit);

			// set the animation sequences for skelaton
			Load_Animation_BOB(&enemy[x],direction,2,enemy_anims[direction]);

			} // end for direction

		// set up stating state of skelaton
		Set_Animation_BOB(&enemy[x], 4);
		Set_Anim_Speed_BOB(&enemy[x], 8);
		Set_Vel_BOB(&enemy[x], 0,0);
		Set_Pos_BOB(&enemy[x], enemy[x].startX, enemy[x].startY); // right above the floor
		enemy[x].direction = -3;
	}
	if(enemy[x].type == 1)
	{
		// create skelaton bob
		if (!Create_BOB(&enemy[x],0,0,72,74,32, // 56x72
				   BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_ANIM,DDSCAPS_SYSTEMMEMORY))
		   return(0);

		// load the frames in 8 directions, 4 frames each
		// each set of frames has a walk and a fire, frame sets
		// are loaded in counter clockwise order looking down
		// from a birds eys view or the x-z plane
		for (direction = 0; direction < 8; direction++)
			{ 
			// build up file name
			sprintf(filename,"IMAGES/bird%d.bmp",direction); // skelsp

			enemy[x].height = 30;
			enemy[x].width = 43;

			// load in new bitmap file
			Load_Bitmap_File(&bitmap8bit,filename);
			Load_Frame_BOB(&enemy[x],&bitmap8bit,0+direction*4,0,0,BITMAP_EXTRACT_MODE_CELL);  
			Load_Frame_BOB(&enemy[x],&bitmap8bit,1+direction*4,1,0,BITMAP_EXTRACT_MODE_CELL);  
			Load_Frame_BOB(&enemy[x],&bitmap8bit,2+direction*4,2,0,BITMAP_EXTRACT_MODE_CELL);  
			Load_Frame_BOB(&enemy[x],&bitmap8bit,3+direction*4,0,1,BITMAP_EXTRACT_MODE_CELL);  

			// unload the bitmap file
			Unload_Bitmap_File(&bitmap8bit);

			// set the animation sequences for skelaton
			Load_Animation_BOB(&enemy[x],direction,3,enemy_anims[direction]);
			

			} // end for direction

		// set up stating state of skelaton
		Set_Animation_BOB(&enemy[x],1);
		Set_Anim_Speed_BOB(&enemy[x], 4);
		Set_Vel_BOB(&enemy[x], 0,0);
		Set_Pos_BOB(&enemy[x], enemy[x].startX, enemy[x].startY); // right above the floor
		enemy[x].direction = -3;
	}
	if(enemy[x].type == 2)
	{
		// create skelaton bob
		if (!Create_BOB(&enemy[x],0,0,72,74,32, // 56x72
				   BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_ANIM,DDSCAPS_SYSTEMMEMORY))
		   return(0);

		// load the frames in 8 directions, 4 frames each
		// each set of frames has a walk and a fire, frame sets
		// are loaded in counter clockwise order looking down
		// from a birds eys view or the x-z plane
		for (direction = 0; direction < 8; direction++)
			{ 
			// build up file name
			sprintf(filename,"IMAGES/shit.bmp",direction); // skelsp

			enemy[x].height = 16;
			enemy[x].width = 18;

			// load in new bitmap file
			Load_Bitmap_File(&bitmap8bit,filename);
			Load_Frame_BOB(&enemy[x],&bitmap8bit,0+direction*4,0,0,BITMAP_EXTRACT_MODE_CELL);  
			Load_Frame_BOB(&enemy[x],&bitmap8bit,1+direction*4,1,0,BITMAP_EXTRACT_MODE_CELL);  
			Load_Frame_BOB(&enemy[x],&bitmap8bit,2+direction*4,2,0,BITMAP_EXTRACT_MODE_CELL);  
			Load_Frame_BOB(&enemy[x],&bitmap8bit,3+direction*4,0,1,BITMAP_EXTRACT_MODE_CELL);  

			// unload the bitmap file
			Unload_Bitmap_File(&bitmap8bit);

			// set the animation sequences for skelaton
			Load_Animation_BOB(&enemy[x],direction,3,enemy_anims[direction]);
			

			} // end for direction

		// set up stating state of skelaton
		Set_Animation_BOB(&enemy[x],1);
		Set_Anim_Speed_BOB(&enemy[x], 4);
		Set_Vel_BOB(&enemy[x], 0,0);
		Set_Pos_BOB(&enemy[x], enemy[x].startX, enemy[x].startY); // right above the floor
		enemy[x].direction = -3;
	}
	if(enemy[x].type == 3)
	{
		// create skelaton bob
		if (!Create_BOB(&enemy[x],0,0,72,74,32, // 56x72
				   BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_ANIM,DDSCAPS_SYSTEMMEMORY))
		   return(0);

		// load the frames in 8 directions, 4 frames each
		// each set of frames has a walk and a fire, frame sets
		// are loaded in counter clockwise order looking down
		// from a birds eys view or the x-z plane
		for (direction = 0; direction < 8; direction++)
			{ 
			// build up file name
			sprintf(filename,"IMAGES/frog%d.bmp",direction); // skelsp

			enemy[x].height = 69;
			enemy[x].width = 69;

			// load in new bitmap file
			Load_Bitmap_File(&bitmap8bit,filename);
			Load_Frame_BOB(&enemy[x],&bitmap8bit,0+direction*4,0,0,BITMAP_EXTRACT_MODE_CELL);  
			Load_Frame_BOB(&enemy[x],&bitmap8bit,1+direction*4,1,0,BITMAP_EXTRACT_MODE_CELL);  
			Load_Frame_BOB(&enemy[x],&bitmap8bit,2+direction*4,2,0,BITMAP_EXTRACT_MODE_CELL);  
			Load_Frame_BOB(&enemy[x],&bitmap8bit,3+direction*4,0,1,BITMAP_EXTRACT_MODE_CELL);  

			// unload the bitmap file
			Unload_Bitmap_File(&bitmap8bit);

			// set the animation sequences for skelaton
			Load_Animation_BOB(&enemy[x],direction,3,enemy_anims[direction]);
			

			} // end for direction

		// set up stating state of skelaton
		Set_Animation_BOB(&enemy[x],1);
		Set_Anim_Speed_BOB(&enemy[x], 4);
		Set_Vel_BOB(&enemy[x], 0,0);
		Set_Pos_BOB(&enemy[x], enemy[x].startX, enemy[x].startY); // right above the floor
		enemy[x].direction = 0;
	}
	if(enemy[x].type == 4)
	{
		// create skelaton bob
		if (!Create_BOB(&enemy[x],0,0,72,74,32, // 56x72
				   BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_ANIM,DDSCAPS_SYSTEMMEMORY))
		   return(0);

		// load the frames in 8 directions, 4 frames each
		// each set of frames has a walk and a fire, frame sets
		// are loaded in counter clockwise order looking down
		// from a birds eys view or the x-z plane
		for (direction = 0; direction < 8; direction++)
			{ 
			// build up file name
			sprintf(filename,"IMAGES/toxic.bmp",direction); // skelsp

			enemy[x].height = 16;
			enemy[x].width = 18;

			// load in new bitmap file
			Load_Bitmap_File(&bitmap8bit,filename);
			Load_Frame_BOB(&enemy[x],&bitmap8bit,0+direction*4,0,0,BITMAP_EXTRACT_MODE_CELL);  
			Load_Frame_BOB(&enemy[x],&bitmap8bit,1+direction*4,1,0,BITMAP_EXTRACT_MODE_CELL);  
			Load_Frame_BOB(&enemy[x],&bitmap8bit,2+direction*4,2,0,BITMAP_EXTRACT_MODE_CELL);  
			Load_Frame_BOB(&enemy[x],&bitmap8bit,3+direction*4,0,1,BITMAP_EXTRACT_MODE_CELL);  

			// unload the bitmap file
			Unload_Bitmap_File(&bitmap8bit);

			// set the animation sequences for skelaton
			Load_Animation_BOB(&enemy[x],direction,3,enemy_anims[direction]);
			

			} // end for direction

		// set up stating state of skelaton
		Set_Animation_BOB(&enemy[x],1);
		Set_Anim_Speed_BOB(&enemy[x], 4);
		Set_Vel_BOB(&enemy[x], 0,0);
		Set_Pos_BOB(&enemy[x], enemy[x].startX, enemy[x].startY); // right above the floor
		enemy[x].direction = -3;
	}
	
		enemy[x].health = enemy[x].startHealth;
}


// set clipping rectangle to screen extents so mouse cursor
// doens't mess up at edges
RECT screen_rect = {0,0,screen_width,screen_height};
lpddclipper = DDraw_Attach_Clipper(lpddsback,1,&screen_rect);
curr_page = STARTPAGE;
// hide the mouse
ShowCursor(FALSE);


// return success
return(1);

} // end Game_Init

///////////////////////////////////////////////////////////

int Game_Shutdown(void *parms,  int num_parms)
{
// this function is where you shutdown your game and
// release all resources that you allocated

// kill the landscape
for (int index=0; index<3; index++)
    Destroy_Bitmap(&landscape[index]);

// kill skelaton
Destroy_BOB(&skelaton);

// shutdonw directdraw
DDraw_Shutdown();

// return success
return(1);
} // end Game_Shutdown

///////////////////////////////////////////////////////////

int Game_Main(void *parms, int num_parms)
{
// this is the workhorse of your game it will be called
// continuously in real-time this is like main() in C
// all the calls for you game go here!

int          index;             // looping var
int          dx,dy;             // general deltas used in collision detection

static int   player_moving = 0; // tracks player motion
bool player_shooting = false;
static PALETTEENTRY glow = {0,0,0,PC_NOCOLLAPSE};  // used to animation green border


// check of user is trying to exit
if (KEY_DOWN(VK_ESCAPE))
    PostMessage(main_window_handle, WM_DESTROY,0,0);

// start the timing clock
Start_Clock();

// clear the drawing surface
DDraw_Fill_Surface(lpddsback, 0);

// lock the back buffer
DDraw_Lock_Back_Surface();

// draw the background reactor image
Draw_Bitmap(&landscape[curr_page], back_buffer, back_lpitch, 0);

// unlock the back buffer
DDraw_Unlock_Back_Surface();

// reset motion flag
player_moving = 0;

if(KEY_DOWN(VK_F1) && reset_help) 
{
	show_help = !show_help;
	reset_help = false;

}
	if(show_help && skelaton.health > 0)
	{
		Draw_Text_GDI("PRESS LEFT/RIGHT ARROWS TO MOVE.",200,215,RGB(200,200,200),lpddsback);
		
		Draw_Text_GDI("PRESS UP TO JUMP.",200,230, RGB(200,200,200),lpddsback);
		
		Draw_Text_GDI("PRESS SPACE BAR TO ATTACK",200,245,RGB(200,200,200),lpddsback);
		
	}
if(KEY_UP(VK_F1))
{
	reset_help = true;
}
	if (KEY_DOWN(VK_F2)) //F2 starts the game over
	{
		for(int x = 0; x < TOTAL_ENEMIES; x++) //reset all enemies to start x, y and health
		{
			enemy[x].x = enemy[x].startX;
			enemy[x].y = enemy[x].startY;
			enemy[x].health = enemy[x].startHealth;
		}
		curr_page = STARTPAGE;  //set the player to the first page
		skelaton.health = 1; //give the player his health back
		skelaton.x = 16;
		skelaton.y = 220;
		game_over_msg = 0;
		checkpoint_page = 0;
		player_lives = 3;
		skelaton.jump = 0;
		for(x = 0; x < TOTAL_BULLETS; x++) //clear all the bullets
		{
			bullet[x].health = 0;
		}
	}
	if (KEY_DOWN(VK_RETURN) && skelaton.health <= 0 && player_lives >0)
	{
		for(int x = 0; x < TOTAL_ENEMIES; x++) //reset all enemies to start x, y and health
		{
			enemy[x].x = enemy[x].startX;
			enemy[x].y = enemy[x].startY;
			enemy[x].health = enemy[x].startHealth;
		}
		curr_page = checkpoint_page;  //set the player to the first page
		skelaton.health = 1; //give the player his health back
		skelaton.x = 16;
		skelaton.y = 120;
		game_over_msg = 0;
		player_lives--;
		skelaton.jump = 0;
		for(x = 0; x < TOTAL_BULLETS; x++) //clear all the bullets
		{
			bullet[x].health = 0;
		}
	}								 
if(skelaton.health > 0) //main part of the game loop, only runs if the player is alive
{
	bool bullet_found = false; //Set true when a unused bullet is found
	int x = 0; // loop counter
	if (KEY_DOWN(VK_SPACE))
	{
		if(shoot_timer == 0){
			while(!bullet_found ) //if we havnt found a bullet yet
			{
				if(bullet[x].health == 0 && bullet_reset == true) //check the next bullet and see if the reset timer has reset
				{
					bullet[x].health = 1; // set bullet to active
					if (!skelaton.curr_animation != SKELATON_EAST) //determine players face direction
						bullet[x].jump = 1; 
					else
						bullet[x].jump = -1;

					bullet[x].x = skelaton.x + 35; //center bullet on player
					bullet[x].y = skelaton.y + skelaton.height/2; //center bullet on player
					bullet_found = true; //bullet has been found :)
					bullet_reset = false; //reset bullet timer
					player_shooting = true;
					shoot_timer = SHOOT_DELAY;
					//PlaySound("sounds/pewpew.wav", main_instance, SND_ASYNC);
					
				}
				if(x == TOTAL_BULLETS) //if all bullets are being used
					bullet_found = true; //lie and say the bullet is found to get out of the loop... you cant shoot anything.
				x++;// increment loop counter
				
			}
		}
	}
	int	y = 0;
	bullet_found = false;
	for(x = 0; x < TOTAL_ENEMIES; x++)
	{
		if(enemy[x].type == 3 && enemy[x].page == curr_page)
		{
			
			if(boss_shoot_timer == 0)
			{
				while(!bullet_found) //if we havnt found a bullet yet
				{
					if(enemy[x + y].health == 0) //check the next bullet and see if the reset timer has reset
					{
						enemy[x + y].health = 1;

						enemy[x + y].x = enemy[x].x; //center bullet on player
						enemy[x + y].y = enemy[x].y + enemy[x].height/2; //center bullet on player
						bullet_found = true; //bullet has been found :)
						
						boss_shoot_timer = SHOOT_DELAY;
						//PlaySound("sounds/pewpew.wav", main_instance, SND_ASYNC);
						
					}
					if(y == BOSS_BULLETS) //if all bullets are being used
						bullet_found = true; //lie and say the bullet is found to get out of the loop... you cant shoot anything.
					y++;// increment loop counter
					
				}
			}
			
		}
	}
	if(boss_shoot_timer != 0)
	{
		boss_shoot_timer--;
	}

	if(shoot_timer != 0){
		shoot_timer--;
	}

	

	if (KEY_UP(VK_SPACE))
	{
		bullet_reset = true;
	}


	// test direction of motion
	if (KEY_DOWN(VK_RIGHT)) 
	   {
	   DDraw_Lock_Back_Surface();
		// move skelaton if there isn't a white color to players right
		if(!Color_Scan(skelaton.x+30, skelaton.y,
				   skelaton.x+skelaton.width-22, skelaton.y+skelaton.height,                                    
				   255, 255, back_buffer,back_lpitch)){

		   skelaton.x+=HORIZONTAL_SPEED;
		   dx=4; dy=0;

		   // set motion flag
		   if(skelaton.jump == 0){player_moving = 1;}

		   // check animation needs to change
		   if (skelaton.curr_animation != SKELATON_EAST)
			  Set_Animation_BOB(&skelaton,SKELATON_EAST);
		}

	   DDraw_Unlock_Back_Surface();

	} // end if
	else
	if (KEY_DOWN(VK_LEFT))  
	   {
	   // move skelaton
		DDraw_Lock_Back_Surface();
		// move skelaton if there isn't a white color to players right
		if(!Color_Scan(skelaton.x+20, skelaton.y,
				   skelaton.x+skelaton.width-35, skelaton.y+skelaton.height,                                    
				   255, 255, back_buffer,back_lpitch)){

		   skelaton.x-=HORIZONTAL_SPEED;
		   dx=-4; dy=0; 
   
		   // set motion flag
		   if(skelaton.jump == 0){player_moving = 1;}

		   // check animation needs to change
		   if (skelaton.curr_animation != SKELATON_WEST)
			  Set_Animation_BOB(&skelaton,SKELATON_WEST);

		   }
		}// end if

	// player is jumping if skelaton.jump is > 0
	// as skelaton.jump gets closer to 0 the player
	// slows down in their ascent.  When skelaton.jump
	// is 0 again the GRAVITY constant takes over and
	// lowers player to the ground.  You can change GRAVITY
	// and JUMP_TIMER in the definition section
	if(skelaton.jump == 0){skelaton.y+=GRAVITY;}
	if(skelaton.jump > (JUMP_TIMER/2)){
		skelaton.y-=GRAVITY;
		skelaton.jump--;
	}
	if(skelaton.jump > (JUMP_TIMER/4) && skelaton.jump <= (JUMP_TIMER/2)){
		skelaton.y-=(GRAVITY/2);
		skelaton.jump--;
	}
	if(skelaton.jump > 0 && skelaton.jump <= (JUMP_TIMER/4)){
		skelaton.jump--;
	}
	
	for(x = 0; x < TOTAL_ENEMIES; x++)
	{
		if(enemy[x].type == 3)
		{
			if(enemy[x].jump == 0){enemy[x].y+=GRAVITY;}
			if(enemy[x].jump > (JUMP_TIMER/2)){
				enemy[x].y-=GRAVITY * 2;
				enemy[x].jump--;
			}
			if(enemy[x].jump > (JUMP_TIMER/4) && enemy[x].jump <= (JUMP_TIMER/2)){
				enemy[x].y-=(GRAVITY/2);
				enemy[x].jump--;
			}
			if(enemy[x].jump > 0 && enemy[x].jump <= (JUMP_TIMER/4)){
				enemy[x].jump--;
			}
		}
	}


	for(x = 0; x < TOTAL_ENEMIES; x++) 
	{	
		if(enemy[x].page == curr_page && enemy[x].type == 2)
		{
			if(enemy[x-1].x - skelaton.x < 200 && enemy[x-1].x - skelaton.x > 30 && enemy[x].health == 0 && enemy[x-1].health > 0)
			{
				enemy[x].x = enemy[x-1].x;
				enemy[x].y = enemy[x-1].y;
				enemy[x].health = 1;
			}
		}

		
		if(enemy[x].page == curr_page && enemy[x].health > 0) //for each alive enemy on the current page
		{
			if(enemy[x].type != 1)
			{
				if(enemy[x].type == 2)
				{
					enemy[x].y += 9; //bombs from birds have more gravity
					if(enemy[x].y > 500)
					{
						enemy[x].health = 0;
					}
				}
				else
					if(enemy[x].type != 4)
						enemy[x].y  += 4; //apply gravity

			}
			else //enemy is 1
			{
				if(enemy[x].x + enemy[x].width < 0)
					enemy[x].x = 600 + enemy[x].width;
			}
			
			enemy[x].x += enemy[x].direction; //move enemy
		}

	}
	// only animate if player is moving
	if (player_moving)
	   {
	   // animate skelaton
	   Animate_BOB(&skelaton);
	   } //end if

	DDraw_Lock_Back_Surface();
	
	for(x = 0; x < TOTAL_BULLETS; x++){
		if(Color_Scan(bullet[x].x, bullet[x].y,
				   bullet[x].x+bullet[x].width, bullet[x].y+bullet[x].height,                                    
				   255, 255, back_buffer,back_lpitch)){
			bullet[x].health--;

		}
	}

	DDraw_Unlock_Back_Surface();

	for( y = 0; y < TOTAL_ENEMIES; y++)
	{
		for(x = 0; x < TOTAL_BULLETS; x++)
		{
			if(bullet[x].health > 0) 
			{
				Animate_BOB(&bullet[x]);
				if(enemy[y].health > 0 && enemy[y].page == curr_page)
					if(bullet[x].x >= enemy[y].x && bullet[x].x < enemy[y].x + enemy[y].width && bullet[x].y 
						+ bullet[x].height >= enemy[y].y && bullet[x].y <= enemy[y].y + enemy[y].height) //bullet and enemy[y] collision detection
					{
						enemy[y].health--;
						
						bullet[x].health--;
					}
			}
		}

		if(enemy[y].health > 0 && enemy[y].page == curr_page && enemy[y].type == 2)
			if(enemy[y].x >= skelaton.x && enemy[y].x <= skelaton.x + skelaton.width - 35 && skelaton.y 
							+ skelaton.height >= enemy[y].y && skelaton.y <= enemy[y].y + enemy[y].height) //check for enemy/skelaton collision
			{
				skelaton.health = 0;
				game_over_msg = 1;
			}
		if(enemy[y].health > 0 && enemy[y].page == curr_page && enemy[y].type == 4)
			if(enemy[y].x >= skelaton.x + 10 && enemy[y].x <= skelaton.x + skelaton.width - 35 && skelaton.y 
							+ skelaton.height >= enemy[y].y && skelaton.y <= enemy[y].y + enemy[y].height) //check for enemy/skelaton collision
			{
				skelaton.health = 1;
				game_over_msg  = 0;
			}
			
		if(enemy[y].health > 0 && enemy[y].page == curr_page && enemy[y].type != 2 && enemy[x].type != 4)
			if(skelaton.x + 35>= enemy[y].x && skelaton.x <= enemy[y].x + 35  && skelaton.y 
							+ skelaton.height >= enemy[y].y && skelaton.y <= enemy[y].y + enemy[y].height) //check for enemy/skelaton collision
			{
				skelaton.health = 0;
				if(enemy[y].type == 0)
					game_over_msg = 2;
				else if(enemy[y].type == 2)
					game_over_msg = 4;
			}
			

		if(skelaton.y > 480)
		{
			skelaton.health = 0;
			game_over_msg = 3;
		}
		if(enemy[y].page == curr_page)
		{
			enemy[y].curr_animation = 5;
			Animate_BOB(&enemy[y]);
		}
	}



	// lock surface, so we can scan it
	DDraw_Lock_Back_Surface();

	
	// call the color scanner with FLOOR_COLOR, the color of the glowing floor
	// try to center the scan on the feet of the player, make sure feet stay
	// in contact with floor
		
		if(Color_Scan(skelaton.x+30, skelaton.y-1.5,
				   skelaton.x+30, skelaton.y,                                    
				   FLOOR_COLOR, FLOOR_COLOR, back_buffer,back_lpitch)){
			skelaton.jump = 0;

		}

		while(Color_Scan(skelaton.x+30, skelaton.y,
				   skelaton.x+skelaton.width-35, skelaton.y+skelaton.height,                                    
				   FLOOR_COLOR, FLOOR_COLOR, back_buffer,back_lpitch))
		{
		// push the skelaton upward, to keep it on the floor
				
					skelaton.y-=1;
		if((KEY_DOWN(VK_UP))){
			   if(skelaton.jump == 0){skelaton.jump = JUMP_TIMER;}
			   //PlaySound("sounds/boing.wav", main_instance, SND_ASYNC);
		   }
	
		   } // end while
	

	//Enemy floor scan
	for(x = 0; x < TOTAL_ENEMIES; x++)
	{
		if(enemy[x].page == curr_page)	
		while(Color_Scan(enemy[x].x+30, enemy[x].y + 4,
					   enemy[x].x+enemy[x].width-35, enemy[x].y+enemy[x].height,                                    
					   FLOOR_COLOR, FLOOR_COLOR, back_buffer,back_lpitch))
		   {
		   // push the enemy[x] upward, to keep it on the floor
   
		   enemy[x].y-=1;
		   if(enemy[x].type == 3)
			   enemy[x].jump = JUMP_TIMER;
		}
		//if(Color_Scan(enemy[x].x+20, enemy[x].y,
		//		   enemy[x].x+enemy[x].width-35, enemy[x].y+enemy[x].height,                                    
		//		   255, 255, back_buffer,back_lpitch))
		//{
		//	enemy[x].direction *= -1;
			//Set_Animation_BOB(&enemy[x], 1);
			
		//}
	}
	// done, so unlock
	DDraw_Unlock_Back_Surface();
}	



for(int x = 0; x < TOTAL_ENEMIES; x++)
	//if enemy is alive, draw it
	if(enemy[x].health > 0 && enemy[x].page == curr_page)
		Draw_BOB(&enemy[x], lpddsback);

// draw the skelaton
if(player_shooting)
	if(skelaton.curr_animation == SKELATON_EAST)
		skelaton.curr_frame = 5;
	else
		skelaton.curr_frame = 6;


	Draw_BOB(&skelaton, lpddsback);

//Check to see which bullets need to be drawn
for(int bullet_count = 0; bullet_count < TOTAL_BULLETS; bullet_count++)
{
	if(bullet[bullet_count].health > 0) //if bullet alive
	{
		bullet[bullet_count].x = bullet[bullet_count].x + BULLET_SPEED * bullet[bullet_count].jump; //move bullet
		Draw_BOB(&bullet[bullet_count], lpddsback); //draw bullet
	}
	if(bullet[bullet_count].x > 640 || bullet[bullet_count].x < 0 - bullet[bullet_count].width) //check to see if bullet is off screen
		bullet[bullet_count].health = 0; //bullet is off screen, deactivate it
}


// this performs the animation for the glowing rock
static int glow_count = 0;

// increment action counter and test
if (++glow_count > 5)
   {
   // rotate the colors
   Rotate_Colors(152,159);
   // reset the counter
   glow_count = 0;
   } // end if


if(skelaton.health < 1)
{
	if(game_over_msg == 0){
		Draw_Text_GDI("GAME OVER!",100,200,RGB(255,255,255),lpddsback);
		//PlaySound("sounds/gameover.wav", main_instance, SND_ASYNC);
	}
	if(game_over_msg == 1){
		Draw_Text_GDI("DON'T LET THE BIRD DOO DOO ON YOUR HEAD! ",100,200,RGB(255,255,255),lpddsback);
		//PlaySound("sounds/doodoo.wav", main_instance, SND_ASYNC);
	}
	if(game_over_msg == 2){
		Draw_Text_GDI("MUTANT SQUIRREL > MUTANT BUNNY! ",150,200,RGB(255,255,255),lpddsback);
		//PlaySound("sounds/squirrels.wav", main_instance, SND_ASYNC);
	}
	if(game_over_msg == 3){
		Draw_Text_GDI("MUTANT BUNNIES CAN'T SWIM!",200,200,RGB(255,255,255),lpddsback);
		//PlaySound("sounds/mutantbunnies.wav", main_instance, SND_ASYNC);
	}
	if(game_over_msg == 4){
		Draw_Text_GDI("THAT BIRD JUST POKED YOUR EYE OUT!",200,200,RGB(255,255,255),lpddsback);
	}
	if(player_lives > 0)
	{
		Draw_Text_GDI("PRESS ENTER TO CONTINUE FROM LAST CHECKPOINT", 115, 220, RGB(255,255,255), lpddsback);
	}
	else
	{
		Draw_Text_GDI("GAME OVER! PRESS F2 TO START A NEW GAME", 125, 220, RGB(255,255,255), lpddsback);

	}
}

Draw_Text_GDI("PRESS F2 TO START A NEW GAME",5,5,RGB(255,255,255),lpddsback);

Draw_Text_GDI("PRESS F1 TO TOGGLE HELP",5,20,RGB(255,255,255),lpddsback);

char* lives_text = "LIVES: ";
int text_pos = 590;

Draw_Text_GDI(lives_text,550,5,RGB(255,255,255),lpddsback);
if(player_lives >= 0)
{
	for(x = 0; x < player_lives; x++)
	{
		Draw_Text_GDI(" X ", text_pos + (x * 8),5, RGB(255,255,255), lpddsback);
	}
}

if(curr_page == 0)
{
	Draw_Text_GDI("EPIC BUNNY RAMPAGE", 200, 100, RGB(255, 0, 0), lpddsback);
	Draw_Text_GDI("You find yourself in a forest waking up beside a pool of toxic waste.", 50, 115, RGB(255, 255, 0), lpddsback);
	Draw_Text_GDI("You can't seem to remember anything prior to waking up.  You see your", 50, 130, RGB(255, 255, 0), lpddsback);
	Draw_Text_GDI("dead rabbit friends besides you and become enraged.  Who could have", 50, 145, RGB(255, 255, 0), lpddsback);
	Draw_Text_GDI("done this?  Why did they die?  Why are you still alive?  You only ", 50, 160, RGB(255, 255, 0), lpddsback);
	Draw_Text_GDI("have moments to think before being attacked by other mutated animals", 50, 175, RGB(255, 255, 0), lpddsback);
	Draw_Text_GDI("in the forest.  You vow to destroy the evil that plagues this forest.", 50, 190, RGB(255, 255, 0), lpddsback);
		
	/*
	You find yourself in a forest waking up beside a pool of toxic waste.
You can't seem to remember anything prior to waking up.  You see your
dead rabbit friends besides you and become enraged.  Who could have
done this?  Why did they die?  Why are you still alive?  You only 
have moments to think before being attacked by other mutated animals
in the forest.  You vow to destroy the evil that plagues this forest.
*/
}

if(curr_page == 17)
{
	Draw_Text_GDI("You've reached the end of the forest.  To be continued...", 50, 190, RGB(255, 255, 0), lpddsback);
	
}


// test for page flip right
if (skelaton.x > SCREEN_WIDTH - (skelaton.width >> 1))
   {
   // bump back regardless
   skelaton.x-=dx;

   // test for page flip
   if (curr_page < (BACKGROUNDS-1))
      { 
      // scroll to next page to right
      curr_page++;

      // reset character to left edge of screen
      skelaton.x = -(skelaton.width >> 1);
	  for(x = 0; x < TOTAL_BULLETS; x++)
		bullet[x].health = 0;
   
      } // end if

   } // end if   
else // page flip right?
if (skelaton.x < -(skelaton.width >> 1))
   {
   // bump back regardless
   skelaton.x=-(skelaton.width >> 1);

   // test for page flip
   if (curr_page > 0 && curr_page != BACKGROUNDS-1)
      { 
      // scroll to next page to left
      curr_page--;
		
      // reset character to right edge of screen
      skelaton.x = SCREEN_WIDTH - (skelaton.width >> 1);
   	  for(x = 0; x < TOTAL_BULLETS; x++)
		bullet[x].health = 0;
      } // end if

   } // end if   

if(curr_page % 4 == 0 && curr_page >= checkpoint_page && curr_page != 0)
{
	checkpoint_page = curr_page;
	
		Draw_Text_GDI("CHECKPOINT!",400,10,RGB(255,255,255),lpddsback);
}

// flip the surfaces
DDraw_Flip();

// sync to 30 fps
Wait_Clock(30);

// return success
return(1);

} // end Game_Main

void newEnemy(int id, int type, int x, int y, int page, int health)
{
	enemy[id].type = type;
	enemy[id].startHealth = health;
	enemy[id].page = page;
	enemy[id].startX = x;
	enemy[id].startY = y;
	if(type == 1)
	{
		enemy[id + 1].startHealth = 0;
		enemy[id + 1].page = page;
		enemy[id + 1].type = 2;
		enemy[id + 1].startX = x;
		enemy[id + 1].startY = y;
	}
	if(type == 3)
	{
		for(int x = 1; x < BOSS_BULLETS + 1; x++)
		{
			enemy[id + x].type = 4;
			enemy[id + x].page = page;
			enemy[id + x].startHealth = 0;
			enemy[id + x].startX = x;
			enemy[id + x].startY = y;

		}
	}
}



//////////////////////////////////////////////////////////